@@ -4,7 +4,7 @@ use 5.10.1;
 use utf8;
 package Term::Choose::GC;
 
-our $VERSION = '0.7.11';
+our $VERSION = '0.7.12';
 use Exporter 'import';
 our @EXPORT_OK = qw(choose);
 
@@ -12,9 +12,9 @@ use Term::Choose qw(choose);
 use Term::ReadKey;
 use Unicode::GCString;
 
-#use warnings FATAL => qw(all);
-#use Log::Log4perl qw(get_logger);
-#my $log = get_logger("Term::Choose");
+use warnings FATAL => qw(all);
+use Log::Log4perl qw(get_logger);
+my $log = get_logger("Term::Choose::GC");
 
 
 use constant {
@@ -39,9 +39,9 @@ sub Term::Choose::_length_longest {
     }
     for my $str ( @{$list} ) {
         eval {
-            my $gcs = Unicode::GCString->new( $str ); 
+            my $gcs = Unicode::GCString->new( $str );
             my $length = $gcs->columns();
-            $longest = $length if $length > $longest;     
+            $longest = $length if $length > $longest;
         };
         if ( $@ ) {
             $longest = length $str if length $str > $longest;
@@ -54,7 +54,7 @@ sub Term::Choose::_length_longest {
 sub Term::Choose::_print_firstline {
     my ( $arg ) = @_;
     $arg->{prompt} =~ s/\p{Space}/ /g;
-    $arg->{prompt} =~ s/\p{Cntrl}//g;      
+    $arg->{prompt} =~ s/\p{Cntrl}//g;
     $arg->{firstline} = $arg->{prompt};
     if ( defined $arg->{wantarray} && $arg->{wantarray} ) {
         if ( $arg->{prompt} ) {
@@ -73,18 +73,18 @@ sub Term::Choose::_print_firstline {
             $arg->{firstline} = '';
         }
     }
-    $arg->{max_length} = $arg->{maxcols} + int( $arg->{maxcols} / 10 );
+    $arg->{step} = 5; # _unicode_cut, _unicode_sprintf
     eval {
         my $gcs = Unicode::GCString->new( $arg->{firstline} );
         if ( $gcs->columns() > $arg->{maxcols} ) {
-            $arg->{firstline} = _unicode_cut( $arg->{prompt}, $arg->{maxcols}, $arg->{max_length} );
+            $arg->{firstline} = _unicode_cut( $arg, $arg->{prompt}, $arg->{maxcols} - 3 ) . '...';
         }
     };
     if ( $@ ) {
         if ( length $arg->{firstline} > $arg->{maxcols} ) {
-            $arg->{firstline} = substr( $arg->{prompt}, 0, $arg->{maxcols} );
+            $arg->{firstline} = substr( $arg->{prompt}, 0, $arg->{maxcols} - 3 ) . '...';
         }
-    }     
+    }
     print $arg->{firstline};
     $arg->{head} = 1;
 }
@@ -92,28 +92,31 @@ sub Term::Choose::_print_firstline {
 
 sub Term::Choose::_wr_cell {
     my( $arg, $row, $col ) = @_;
-    if ( $#{$arg->{new_list}} == 0 ) {
+    if ( $#{$arg->{rowcol2list}} == 0 ) {
         my $lngth = 0;
         if ( $col > 0 ) {
             for my $cl ( 0 .. $col - 1 ) {
                 eval {
-                    my $gcs = Unicode::GCString->new( $arg->{new_list}[$row][$cl] );
+                    my $gcs = Unicode::GCString->new( $arg->{list}[$arg->{rowcol2list}[$row][$cl]] );
                     $lngth += $gcs->columns();
                 };
                 if ( $@ ) {
-                    $lngth += length $arg->{new_list}[$row][$cl];
+                    $lngth += length $arg->{list}[$arg->{rowcol2list}[$row][$cl]];
                 }
                 $lngth += $arg->{pad_one_row} // 0;
             }
         }
         Term::Choose::_goto( $arg, $row + $arg->{head} - $arg->{page}, $lngth );
-    } 
+        print BOLD, UNDERLINE if $arg->{marked}[$row][$col];
+        print REVERSE if [ $row, $col ] ~~ $arg->{this_cell};
+        print $arg->{list}[$arg->{rowcol2list}[$row][$col]];
+    }
     else {
         Term::Choose::_goto( $arg, $row + $arg->{head} - $arg->{page}, $col * $arg->{col_width} );
+        print BOLD, UNDERLINE if $arg->{marked}[$row][$col];
+        print REVERSE if [ $row, $col ] ~~ $arg->{this_cell};
+        print _unicode_sprintf( $arg, $arg->{list}[$arg->{rowcol2list}[$row][$col]] );
     }
-    print BOLD, UNDERLINE if $arg->{marked}[$row][$col];
-    print REVERSE if [ $row, $col ] ~~ $arg->{this_cell};
-    print $arg->{new_list}[$row][$col];
     print RESET if $arg->{marked}[$row][$col] || [ $row, $col ] ~~ $arg->{this_cell};
 }
 
@@ -121,18 +124,17 @@ sub Term::Choose::_wr_cell {
 sub Term::Choose::_size_and_layout {
     my ( $arg ) = @_;
     my $layout = $arg->{layout};
-    $arg->{new_list} = [];
-    $arg->{rowcol_to_list_index} = [];
+    $arg->{rowcol2list} = [];
     $arg->{all_in_first_row} = 0;
     if ( $arg->{length_longest} > $arg->{maxcols} ) {
         $arg->{length_longest} = $arg->{maxcols};
-        $layout = 2;
+        $layout = 3;
     }
-    # layout
+    ### layout
     $arg->{this_cell} = [ 0, 0 ];
     my $all_in_first_row;
-    if ( $layout == 3 ) {
-        $layout = 2 if scalar @{$arg->{list}} <= $arg->{maxrows};
+    if ( $layout == 2 ) {
+        $layout = 3 if scalar @{$arg->{list}} <= $arg->{maxrows};
     }
     elsif ( $layout < 2 ) {
         for my $idx ( 0 .. $#{$arg->{list}} ) {
@@ -154,15 +156,14 @@ sub Term::Choose::_size_and_layout {
     }
     if ( $all_in_first_row ) {
         $arg->{all_in_first_row} = 1;
-        $arg->{new_list}[0] = [ @{$arg->{list}} ];
-        $arg->{rowcol_to_list_index}[0] = [ 0 .. $#{$arg->{list}} ];    
+        $arg->{rowcol2list}[0] = [ 0 .. $#{$arg->{list}} ];
     }
-    elsif ( $layout == 2 ) {
+    elsif ( $layout == 3 ) {
         for my $idx ( 0 .. $#{$arg->{list}} ) {
-            eval {
-                my $gcs = Unicode::GCString->new( $arg->{list}[$idx] );            
+        	eval {
+                my $gcs = Unicode::GCString->new( $arg->{list}[$idx] );
                 if ( $gcs->columns() > $arg->{length_longest} ) {
-                    $arg->{list}[$idx] = _unicode_cut( $arg->{list}[$idx], $arg->{length_longest} - 3, $arg->{max_length} ) . '...';
+                    $arg->{list}[$idx] = _unicode_cut( $arg, $arg->{list}[$idx], $arg->{length_longest} - 3 ) . '...';
                 }
             };
             if ( $@ ) {
@@ -170,65 +171,51 @@ sub Term::Choose::_size_and_layout {
                     $arg->{list}[$idx] = substr( $arg->{list}[$idx], 0, $arg->{length_longest} - 3 ) . '...';
                 }
             }
-            #$arg->{new_list}[$idx][0] = _unicode_sprintf( $arg->{length_longest}, $arg->{list}[$idx], $arg->{right_justify}, $arg->{max_length} );
-            $arg->{new_list}[$idx][0] = _unicode_sprintf( $arg->{length_longest}, $arg->{list}[$idx], $arg->{right_justify}, undef );            
-            $arg->{rowcol_to_list_index}[$idx][0] = $idx;
+            $arg->{rowcol2list}[$idx][0] = $idx;
         }
     }
     else {
         # auto_format
         my $maxcls = $arg->{maxcols};
-        if ( ( $arg->{layout} == 1 || $arg->{layout} == 3 ) && $arg->{maxrows} > 0 ) {
+        if ( ( $arg->{layout} == 1 || $arg->{layout} == 2 ) && $arg->{maxrows} > 0 ) {
             my $tmc = int( @{$arg->{list}} / $arg->{maxrows} );
             $tmc++ if @{$arg->{list}} % $arg->{maxrows};
             $tmc *= $arg->{col_width};
             if ( $tmc < $maxcls ) {
                 $tmc = int( $tmc + ( ( $maxcls - $tmc ) / 2 ) ) if $arg->{layout} == 1;
-                $tmc = int( $tmc + ( ( $maxcls - $tmc ) / 6 ) ) if $arg->{layout} == 3;
+                $tmc = int( $tmc + ( ( $maxcls - $tmc ) / 6 ) ) if $arg->{layout} == 2;
                 $maxcls = $tmc;
             }
         }
-        # end auto_format
-    # end layout
-    # row_first
+    ### vertical
         my $cols_per_row = int( $maxcls / $arg->{col_width} );
         $cols_per_row = 1 if $cols_per_row < 1;
         my $rows = int( ( $#{$arg->{list}} + $cols_per_row ) / $cols_per_row );
         $arg->{rest} = @{$arg->{list}} % $cols_per_row;
-        if ( $arg->{vertical_order} ) {
-            my @rearranged_list;
+        if ( $arg->{vertical} ) {
             my @rearranged_idx;
-            my $i = 0;
-            my $idxs = [ 0 .. $#{$arg->{list}} ];
+            my $begin = 0;
+            my $end = $rows - 1;
             for my $c ( 0 .. $cols_per_row - 1 ) {
-                $i = 1 if $arg->{rest} && $c >= $arg->{rest};
-                $rearranged_list[$c] = [ splice( @{$arg->{list}}, 0, $rows - $i ) ];
-                $rearranged_idx[$c]  = [ splice( @{$idxs},        0, $rows - $i ) ];
+                --$end if $arg->{rest} && $c >= $arg->{rest};
+                $rearranged_idx[$c]  = [ $begin .. $end ];
+                $begin = $end + 1;
+                $end = $begin + $rows - 1;
             }
             for my $r ( 0 .. $rows - 1 ) {
-                my @temp_new_list;
                 my @temp_idx;
                 for my $c ( 0 .. $cols_per_row - 1 ) {
                     next if $arg->{rest} && $r == $rows - 1 && $c >= $arg->{rest};
-                    # push @temp_new_list, _unicode_sprintf( $arg->{length_longest}, $rearranged_list[$c][$r], $arg->{right_justify}, $arg->{max_length} );
-                    push @temp_new_list, _unicode_sprintf( $arg->{length_longest}, $rearranged_list[$c][$r], $arg->{right_justify}, undef );                    
                     push @temp_idx, $rearranged_idx[$c][$r];
                 }
-                push @{$arg->{new_list}}, \@temp_new_list;
-                push @{$arg->{rowcol_to_list_index}}, \@temp_idx;
+                push @{$arg->{rowcol2list}}, \@temp_idx;
             }
         }
         else {
             my $begin = 0;
             my $end = $cols_per_row - 1;
-            while ( my @rearranged_list = @{$arg->{list}}[$begin..$end] ) {
-                my @temp_new_list;
-                for my $rearranged_list_item ( @rearranged_list ) {
-                    # push @temp_new_list, _unicode_sprintf( $arg->{length_longest}, $rearranged_list_item, $arg->{right_justify}, $arg->{max_length} );
-                    push @temp_new_list, _unicode_sprintf( $arg->{length_longest}, $rearranged_list_item, $arg->{right_justify}, undef );                    
-                }
-                push @{$arg->{new_list}}, \@temp_new_list;
-                push @{$arg->{rowcol_to_list_index}}, [ $begin .. $end ];
+            while ( @{$arg->{list}}[$begin..$end] ) { ###
+                push @{$arg->{rowcol2list}}, [ $begin .. $end ];
                 $begin = $end + 1;
                 $end = $begin + $cols_per_row - 1;
                 $end = $#{$arg->{list}} if $end > $#{$arg->{list}};
@@ -239,65 +226,75 @@ sub Term::Choose::_size_and_layout {
 
 
 sub _unicode_cut {
-    my ( $unicode, $length, $max_length ) = @_; 
+    my ( $arg, $unicode, $length ) = @_;
     my $gcs = Unicode::GCString->new( $unicode );
     my $colwidth = $gcs->columns();
-        if ( defined $max_length && $colwidth > $max_length ) {
-            $unicode = substr( $gcs->as_string, 0, $max_length );
-            my $gcs = Unicode::GCString->new( $unicode );
-            $colwidth = $gcs->columns();
-        }
-        while ( $colwidth > $length ) {
-            $unicode =~ s/\X\z//;
-            my $gcs = Unicode::GCString->new( $unicode );
-            $colwidth = $gcs->columns();
-        }
-        $unicode .= ' ' if $colwidth < $length;
+    my $max_length = ( $arg->{maxcols} / 2 ) + 1;
+    while ( 1 ) {
+		my $tmp = substr( $unicode, 0, $max_length );
+		my $gcs = Unicode::GCString->new( $tmp );
+		$colwidth = $gcs->columns();
+		if ( $colwidth > $arg->{maxcols} ) {
+			$unicode = $tmp;
+			last;
+		}
+		$max_length += $arg->{step};
+	}
+	while ( $colwidth > $length ) {
+		$unicode =~ s/\X\z//;
+		my $gcs = Unicode::GCString->new( $unicode );
+		$colwidth = $gcs->columns();
+	}
+	$unicode .= ' ' if $colwidth < $length;
     return $unicode;
 }
 
 
 sub _unicode_sprintf {
-    my ( $length, $word, $right_justify, $max_length ) = @_;
+    my ( $arg, $word ) = @_;
     my $unicode = $word;
     eval {
         my $gcs = Unicode::GCString->new( $unicode );
         my $colwidth = $gcs->columns();
-        if ( $colwidth > $length ) {
-            if ( defined $max_length && $colwidth > $max_length ) {
-                $unicode = substr( $gcs->as_string, 0, $max_length );
-                my $gcs = Unicode::GCString->new( $unicode );
-                $colwidth = $gcs->columns();
-            }
-            while ( $colwidth > $length ) {
-                $unicode =~ s/\X\z//;
-                my $gcs = Unicode::GCString->new( $unicode );
-                $colwidth = $gcs->columns();
-            }
-            $unicode .= ' ' if $colwidth < $length;
-        } 
-        #else { 
-        elsif ( $colwidth < $length ) {
-            if ( $right_justify ) {
-                $unicode = " " x ( $length - $colwidth ) . $unicode;
+        if ( $colwidth > $arg->{length_longest} ) {
+			my $max_length = ( $arg->{length_longest} / 2 ) + 1;
+			while ( 1 ) {
+				my $tmp = substr( $unicode, 0, $max_length );
+				my $gcs = Unicode::GCString->new( $tmp );
+				$colwidth = $gcs->columns();
+				if ( $colwidth > $arg->{length_longest} ) {
+					$unicode = $tmp;
+					last;
+				}
+				$max_length += $arg->{step};
+			}
+			while ( $colwidth > $arg->{length_longest} ) {
+				$unicode =~ s/\X\z//;
+				my $gcs = Unicode::GCString->new( $unicode );
+				$colwidth = $gcs->columns();
+			}
+			$unicode .= ' ' if $colwidth < $arg->{length_longest};
+        }
+        elsif ( $colwidth < $arg->{length_longest} ) {
+            if ( $arg->{right_justify} ) {
+                $unicode = " " x ( $arg->{length_longest} - $colwidth ) . $unicode;
             }
             else {
-                $unicode = $unicode . " " x ( $length - $colwidth );
+                $unicode = $unicode . " " x ( $arg->{length_longest} - $colwidth );
             }
         }
     };
     if ( $@ ) {
         my $colwidth = length $word;
-        if ( $colwidth > $length ) {
-            $word = substr( $word, 0, $length );
-        } 
-        #else {
-        elsif ( $colwidth < $length ) {        
-            if ( $right_justify ) {
-                $word = " " x ( $length - $colwidth ) . $word;
+        if ( $colwidth > $arg->{length_longest} ) {
+            $word = substr( $word, 0, $arg->{length_longest} );
+        }
+        elsif ( $colwidth < $arg->{length_longest} ) {
+            if ( $arg->{right_justify} ) {
+                $word = " " x ( $arg->{length_longest} - $colwidth ) . $word;
             }
             else {
-                $word = $word . " " x ( $length - $colwidth );
+                $word = $word . " " x ( $arg->{length_longest} - $colwidth );
             }
         }
         return $word;
@@ -307,6 +304,34 @@ sub _unicode_sprintf {
     }
 }
 
+# without the option "length_longest" this _unicode_sprintf would suffice
+#
+#sub _unicode_sprintf {
+#    my ( $arg, $unicode ) = @_;
+#    my $colwidth;
+#    eval {
+#        my $gcs = Unicode::GCString->new( $unicode );
+#        $colwidth = $gcs->columns();
+#    };
+#    if ( $@ ) {
+#        $colwidth = length $unicode;
+#    }
+#	if ( $colwidth < $arg->{length_longest} ) {
+#		if ( $arg->{right_justify} ) {
+#			$unicode = " " x ( $arg->{length_longest} - $colwidth ) . $unicode;
+#		}
+#		else {
+#			$unicode = $unicode . " " x ( $arg->{length_longest} - $colwidth );
+#		}
+#	}
+#	return $unicode;
+#}
+
+
+
+1;
+
+__END__
 
 =pod
 
@@ -318,7 +343,7 @@ Term::Choose::GC - Works as L<Term::Choose>.
 
 =head1 VERSION
 
-Version 0.7.11
+Version 0.7.12
 
 =cut
 
@@ -334,7 +359,7 @@ Version 0.7.11
 
     my @choices = choose( [ 1 .. 100 ], { right_justify => 1 } ); # multiple choice
     say "@choices";
-    
+
     choose( [ 'Press ENTER to continue' ], { prompt => 0 } );     # no choice
 
 
@@ -351,12 +376,16 @@ See L<Term::Choose> for details.
 Nothing by default.
 
     use Term::Choose::GC qw(choose);
-    
+
 =head1 DIFFERENCES
 
 =head2 UNICODE
 
-While L<Term::Choose> uses the Perl builtin functions I<length> to determine the length of strings and I<sprintf> widths to justify strings L<Term::Choose::GC> uses L<Unicode::GCString::columns|http://search.cpan.org/perldoc?Unicode::GCString#Sizes> to determine the length of strings. To justify strings it uses its own function based on L<Unicode::GCString>. The codeparts using L<Unicode::GCString::columns|http://search.cpan.org/perldoc?Unicode::GCString#Sizes> run in I<eval> blocks: if the code in the eval block fails builtin I<length> resp. I<substr> are used instead. The reason for this procedure with I<eval> is to make L<Term::Choose::GC>'s choose work also with non-unicode characters. 
+While L<Term::Choose> uses the Perl builtin functions I<length> to determine the length of strings and I<sprintf> widths to justify strings L<Term::Choose::GC> uses L<Unicode::GCString::columns|http://search.cpan.org/perldoc?Unicode::GCString#Sizes> to determine the length of strings. To justify strings it uses its own function based on L<Unicode::GCString>. The codeparts using L<Unicode::GCString::columns|http://search.cpan.org/perldoc?Unicode::GCString#Sizes> run in I<eval> blocks: if the code in the eval block fails builtin I<length> resp. I<substr> are used instead. The reason for this procedure with I<eval> is to make L<Term::Choose::GC>'s choose work also with non-unicode characters.
+
+=head4 length_longest
+
+If the option I<length_longest> is set, it is needed to calculate the value from the strings length of print columns which can be achieved by using the I<columns> method from L<Unicode::GCString>.
 
 =head2 REQUIREMENTS
 
@@ -386,7 +415,7 @@ This library is free software; you can redistribute it and/or modify it under th
 
 =cut
 
-1; # End of Term::Choose
+# End of Term::Choose
 
 
 
@@ -405,4 +434,3 @@ This library is free software; you can redistribute it and/or modify it under th
 
 
 
-1;
\ No newline at end of file
